﻿using System.Net.Http;
using System.Threading.Tasks;
using Microsoft.AspNetCore.Mvc;
using Newtonsoft.Json;
using Polly;

namespace CachePolicyInsideHttpClientFactory.Controllers
{
    [Produces("application/json")]
    [Route("api/[controller]")]
    public class CatalogController : Controller
    {
        private readonly IHttpClientFactory _httpClientFactory;

        public CatalogController(IHttpClientFactory httpClientFactory)
        {
            _httpClientFactory = httpClientFactory;
        }

        [HttpGet("{id}")]
        public async Task<IActionResult> Get(int id)
        {
            string requestEndpoint = $"inventory/{id}";
            var httpClient = _httpClientFactory.CreateClient("RemoteServer");

            HttpRequestMessage httpRequestMessage = new HttpRequestMessage(HttpMethod.Get, 
                new System.Uri(httpClient.BaseAddress + requestEndpoint));

            httpRequestMessage.SetPolicyExecutionContext(new Context($"GetInventoryById-{id}"));
            HttpResponseMessage response = await httpClient.SendAsync(httpRequestMessage);

            if (response.IsSuccessStatusCode)
            {
                int itemsInStock = JsonConvert.DeserializeObject<int>(await response.Content.ReadAsStringAsync());
                return Ok(itemsInStock);
            }

            return StatusCode((int)response.StatusCode, response.Content.ReadAsStringAsync());
        }

    }
}
